/*
 * Copyright (c) 2008-2016 Stefan Krah. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <pthread.h>
#include "../mpdecimal.h"




mpd_ssize_t prec[10];
mpd_t *expected[10];

void *
thfunc(void *arg)
{
	mpd_context_t ctx;
	mpd_t *a, *b, *result;
	uint32_t status = 0;
	long threadno = (long)arg;
	int i;

	mpd_defaultcontext(&ctx);
	ctx.prec = prec[threadno];

	a = mpd_new(&ctx);
	b = mpd_new(&ctx);
	mpd_set_string(a, "2", &ctx);
	mpd_set_string(b, "2", &ctx);
	result = mpd_new(&ctx);

	fprintf(stderr, "THREADNO=%ld\n", threadno);
	for (i = 0; i < 10000; i++) {
		mpd_log10(result, a, &ctx);
		mpd_add(b, b, result, &ctx);
		if (mpd_qcmp(result, expected[threadno], &status)) {
			fprintf(stderr, "FAIL\n");
			exit(1);
		}
	}

	mpd_fprint(stderr, b);

	mpd_del(a);
	mpd_del(b);
	mpd_del(result);
	return NULL;
}


int
main(int argc, char **argv)
{
	mpd_context_t ctx;
	mpd_t *a;
	pthread_t tid[10];
	void *tret[10];
	long i;

	mpd_defaultcontext(&ctx);
	prec[0] = 34;
	prec[1] = 28;
	prec[2] = 19;
	prec[3] = 20;

	a = mpd_new(&ctx);

	for (i = 0; i < 4; i++) {
		mpd_set_string(a, "2", &ctx);
		ctx.prec = prec[i];
		expected[i] = mpd_qnew();
		mpd_log10(expected[i], a, &ctx);
		mpd_print(expected[i]);
	}

	for (i = 0; i < 4; i++) {
		if (pthread_create(&tid[i], NULL, thfunc, (void *)i) != 0) {
			fprintf(stderr, "FAIL\n");
			exit(1);
		}
	}

	for (i = 0; i < 4; i++) {
		if (pthread_join(tid[i], &tret[i]) != 0) {
			fprintf(stderr, "FAIL\n");
			exit(1);
		}
	}

	for (i = 0; i < 4; i++) {
		mpd_del(expected[i]);
	}
	mpd_del(a);

	return 0;
}


